@@ -1,18 +1,18 @@
 import logging
 import os
 import plistlib
+import traceback
 
 import IPython
 import click
 from pygments import highlight, lexers, formatters
-
 from pymobiledevice3.cli.cli_common import print_json
 from pymobiledevice3.exceptions import IncorrectModeError
 from pymobiledevice3.irecv import IRecv
 from pymobiledevice3.lockdown import list_devices, LockdownClient
+from pymobiledevice3.restore.device import Device
 from pymobiledevice3.restore.recovery import Recovery
 from pymobiledevice3.restore.restore import Restore
-from pymobiledevice3.restore.restored_client import RestoredClient
 
 SHELL_USAGE = """
 # use `irecv` variable to access Restore mode API
@@ -63,12 +63,12 @@ def restore():
 
 
 @restore.command('shell', cls=Command)
-def restore_shell(irecv):
+def restore_shell(device):
     """ create an IPython shell for interacting with iBoot """
     IPython.embed(
         header=highlight(SHELL_USAGE, lexers.PythonLexer(), formatters.TerminalTrueColorFormatter(style='native')),
         user_ns={
-            'irecv': irecv,
+            'irecv': device,
         })
 
 
@@ -100,7 +100,8 @@ def restore_tss(device, ipsw, out, color, offline):
     elif isinstance(device, IRecv):
         irecv = device
 
-    tss = Recovery(ipsw, lockdown=lockdown, irecv=irecv, offline=offline).fetch_tss_record()
+    device = Device(lockdown=lockdown, irecv=irecv)
+    tss = Recovery(ipsw, device, offline=offline).fetch_tss_record()
     if out:
         plistlib.dump(tss, out)
     print_json(tss, colored=color)
@@ -120,14 +121,16 @@ def restore_ramdisk(device, ipsw, tss):
         lockdown = device
     elif isinstance(device, IRecv):
         irecv = device
-    Recovery(ipsw, lockdown=lockdown, irecv=irecv, tss=tss).boot_ramdisk()
+    device = Device(lockdown=lockdown, irecv=irecv)
+    Recovery(ipsw, device, tss=tss).boot_ramdisk()
 
 
 @restore.command('update', cls=Command)
 @click.argument('ipsw', type=click.File('rb'))
 @click.option('--tss', type=click.File('rb'))
 @click.option('--offline', is_flag=True)
-def restore_update(device, ipsw, tss, offline):
+@click.option('--erase', is_flag=True)
+def restore_update(device, ipsw, tss, offline, erase):
     """ perform an upgrade """
     if tss:
         tss = plistlib.load(tss)
@@ -138,4 +141,15 @@ def restore_update(device, ipsw, tss, offline):
         lockdown = device
     elif isinstance(device, IRecv):
         irecv = device
-    Restore(ipsw, lockdown=lockdown, irecv=irecv, tss=tss, offline=offline).update()
+    device = Device(lockdown=lockdown, irecv=irecv)
+
+    behavior = 'Update'
+    if erase:
+        behavior = 'Erase'
+
+    try:
+        Restore(ipsw, device, tss=tss, offline=offline, behavior=behavior).update()
+    except Exception:
+        # click may "swallow" several exception types so we try to catch them all here
+        traceback.print_exc()
+        raise
