@@ -1,23 +1,28 @@
 import logging
 import plistlib
 import time
+import typing
 from pathlib import Path
 from uuid import uuid4
 
 import asn1
 import requests
-
 from pymobiledevice3.exceptions import PyMobileDevice3Exception
 from pymobiledevice3.restore.img4 import img4_get_component_tag
+from pymobiledevice3.utils import bytes_to_uint
 
 TSS_CLIENT_VERSION_STRING = 'libauthinstall-776.60.1'
-TICKETS_SUBDIR = Path('tickets')
+TICKETS_SUBDIR = Path('offline_requests')
 
 OFFLINE_REQUEST_SCRIPT = """#!/bin/sh
 curl -d "@{request}" -H 'Cache-Control: no-cache' -H 'Content-type: text/xml; charset="utf-8"' -H 'User-Agent: InetURL/1.0' -H 'Expect: ' 'http://gs.apple.com/TSS/controller?action=2' | tee {response} 
 """
 
 
+def get_with_or_without_comma(obj: typing.Mapping, k: str):
+    return obj.get(k, obj.get(k.replace(',', '')))
+
+
 class TSSResponse(dict):
     @property
     def ap_img4_ticket(self):
@@ -51,7 +56,7 @@ class TSSRequest:
         self._offline = offline
 
     @staticmethod
-    def apply_restore_request_rules(tss_entry: dict, parameters: dict, rules: list):
+    def apply_restore_request_rules(tss_entry: typing.Mapping, parameters: typing.Mapping, rules: list):
         for rule in rules:
             conditions_fulfilled = True
             conditions = rule['Conditions']
@@ -89,11 +94,11 @@ class TSSRequest:
                     value2 = tss_entry.get(key)
                     if value2:
                         tss_entry.pop(key)
-                    logging.debug(f'DEBUG: Adding {key}={value} to TSS entry')
+                    logging.debug(f'Adding {key}={value} to TSS entry')
                     tss_entry[key] = value
         return tss_entry
 
-    def add_common_tags(self, parameters: dict, overrides=None):
+    def add_common_tags(self, parameters: typing.Mapping, overrides=None):
         keys = ('ApECID', 'UniqueBuildID', 'ApChipID', 'ApBoardID', 'ApSecurityDomain')
         for k in keys:
             if k in parameters:
@@ -101,7 +106,7 @@ class TSSRequest:
         if overrides is not None:
             self._request.update(overrides)
 
-    def add_local_policy_tags(self, parameters: dict):
+    def add_local_policy_tags(self, parameters: typing.Mapping):
         self._request['@ApImg4Ticket'] = True
 
         keys_to_copy = (
@@ -114,7 +119,7 @@ class TSSRequest:
             if k in parameters:
                 self._request[k] = parameters[k]
 
-    def add_vinyl_tags(self, parameters: dict, overrides=None):
+    def add_vinyl_tags(self, parameters: typing.Mapping, overrides=None):
         self._request['@BBTicket'] = True
         self._request['@eUICC,Ticket'] = True
 
@@ -126,19 +131,21 @@ class TSSRequest:
         # set Nonce for eUICC,Gold component
         node = parameters.get('EUICCGoldNonce')
         if node is not None:
-            n = self._request['eUICC,Gold']
-            n['Nonce'] = node
+            n = self._request.get('eUICC,Gold')
+            if n is not None:
+                n['Nonce'] = node
 
         # set Nonce for eUICC,Main component
         node = parameters.get('EUICCMainNonce')
         if node is not None:
-            n = self._request['eUICC,Main']
-            n['Nonce'] = node
+            n = self._request.get('eUICC,Main')
+            if n is not None:
+                n['Nonce'] = node
 
         if overrides is not None:
             self._request.update(overrides)
 
-    def add_ap_tags(self, parameters: dict, overrides=None):
+    def add_ap_tags(self, parameters: typing.Mapping, overrides=None):
         """ loop over components from build manifest """
 
         manifest_node = parameters['Manifest']
@@ -149,6 +156,14 @@ class TSSRequest:
             if key in skipped_keys:
                 continue
 
+            info_dict = manifest_entry.get('Info')
+            if info_dict is None:
+                continue
+
+            if parameters.get('ApSupportsImg4', False) and ('RestoreRequestRules' not in info_dict):
+                logging.debug(f'Skipping "{key}" as it doesn\'t have RestoreRequestRules')
+                continue
+
             if parameters.get('_OnlyFWComponents', False):
                 if not manifest_node.get('Trusted', False):
                     logging.debug(f'skipping {key} as it is not trusted')
@@ -183,7 +198,7 @@ class TSSRequest:
         if overrides is not None:
             self._request.update(overrides)
 
-    def add_ap_img3_tags(self, parameters: dict):
+    def add_ap_img3_tags(self, parameters: typing.Mapping):
         if 'ApNonce' in parameters:
             self._request['ApNonce'] = parameters['ApNonce']
         self._request['@APTicket'] = True
@@ -201,7 +216,7 @@ class TSSRequest:
 
         self._request['@ApImg4Ticket'] = True
 
-    def add_se_tags(self, parameters: dict, overrides=None):
+    def add_se_tags(self, parameters: typing.Mapping, overrides=None):
         manifest = parameters['Manifest']
 
         # add tags indicating we want to get the SE,Ticket
@@ -256,13 +271,56 @@ class TSSRequest:
         if overrides is not None:
             self._request.update(overrides)
 
-    def add_savage_tags(self, parameters: dict, overrides=None, component_name=None):
-        raise NotImplementedError()
-
-    def add_yonkers_tags(self, parameters: dict, overrides=None):
+    def add_savage_tags(self, parameters: typing.Mapping, overrides=None, component_name=None):
         manifest = parameters['Manifest']
 
-        # add tags indicating we want to get the SE,Ticket
+        # add tags indicating we want to get the Savage,Ticket
+        self._request['@BBTicket'] = True
+        self._request['@Savage,Ticket'] = True
+
+        # add Savage,UID
+        self._request['Savage,UID'] = get_with_or_without_comma(parameters, 'Savage,UID')
+
+        # add SEP
+        self._request['SEP'] = {'Digest': manifest['SEP']['Digest']}
+
+        keys_to_copy = (
+            'Savage,PatchEpoch', 'Savage,ChipID', 'Savage,AllowOfflineBoot', 'Savage,ReadFWKey',
+            'Savage,ProductionMode', 'Savage,Nonce', 'Savage,Nonce')
+
+        for k in keys_to_copy:
+            value = get_with_or_without_comma(parameters, k)
+            if value is None:
+                continue
+            self._request[k] = value
+
+        isprod = get_with_or_without_comma(parameters, 'Savage,ProductionMode')
+
+        # get the right component name
+        comp_name = 'Savage,B0-Prod-Patch' if isprod else 'Savage,B0-Dev-Patch'
+        node = get_with_or_without_comma(parameters, 'Savage,Revision')
+
+        if isinstance(node, bytes):
+            savage_rev = node
+            if ((savage_rev[0] | 0x10) & 0xF0) == 0x30:
+                comp_name = 'Savage,B2-Prod-Patch' if isprod else 'Savage,B2-Dev-Patch'
+            elif (savage_rev[0] & 0xF0) == 0xA0:
+                comp_name = 'Savage,BA-Prod-Patch' if isprod else 'Savage,BA-Dev-Patch'
+
+        # add Savage,B?-*-Patch
+        d = dict(manifest[comp_name])
+        d.pop('Info')
+        self._request[comp_name] = d
+
+        if overrides is not None:
+            self._request.update(overrides)
+
+        return comp_name
+
+    def add_yonkers_tags(self, parameters: typing.Mapping, overrides=None):
+        manifest = parameters['Manifest']
+
+        # add tags indicating we want to get the Yonkers,Ticket
         self._request['@BBTicket'] = True
         self._request['@Yonkers,Ticket'] = True
 
@@ -312,7 +370,7 @@ class TSSRequest:
 
         return result_comp_name
 
-    def add_baseband_tags(self, parameters: dict, overrides=None):
+    def add_baseband_tags(self, parameters: typing.Mapping, overrides=None):
         self._request['@BBTicket'] = True
 
         keys_to_copy = (
@@ -344,7 +402,110 @@ class TSSRequest:
         if overrides:
             self._request.update(overrides)
 
-    def img4_create_local_manifest(self):
+    def add_rose_tags(self, parameters: typing.Mapping, overrides: typing.Mapping = None):
+        manifest = parameters['Manifest']
+
+        # add tags indicating we want to get the Rap,Ticket
+        self._request['@BBTicket'] = True
+        self._request['@Rap,Ticket'] = True
+
+        keys_to_copy_uint = ('Rap,BoardID', 'Rap,ChipID', 'Rap,ECID', 'Rap,SecurityDomain',)
+
+        for key in keys_to_copy_uint:
+            value = get_with_or_without_comma(parameters, key)
+
+            if isinstance(value, bytes):
+                self._request[key] = bytes_to_uint(value)
+            else:
+                self._request[key] = value
+
+        keys_to_copy_bool = ('Rap,ProductionMode', 'Rap,SecurityMode',)
+
+        for key in keys_to_copy_bool:
+            value = get_with_or_without_comma(parameters, key)
+            self._request[key] = bytes_to_uint(value) == 1
+
+        nonce = get_with_or_without_comma(parameters, 'Rap,Nonce')
+
+        if nonce is not None:
+            self._request['Rap,Nonce'] = nonce
+
+        for comp_name, node in manifest.items():
+            if not comp_name.startswith('Rap,'):
+                continue
+
+            manifest_entry = dict(node)
+
+            # handle RestoreRequestRules
+            rules = manifest_entry['Info'].get('RestoreRequestRules')
+            if rules is not None:
+                self.apply_restore_request_rules(manifest_entry, parameters, rules)
+
+            # Make sure we have a Digest key for Trusted items even if empty
+            trusted = manifest_entry.get('Trusted', False)
+
+            if trusted:
+                digest = manifest_entry.get('Digest')
+                if digest is None:
+                    logging.debug(f'No Digest data, using empty value for entry {comp_name}')
+                    manifest_entry['Digest'] = b''
+
+            manifest_entry.pop('Info')
+
+            # finally add entry to request
+            self._request[comp_name] = manifest_entry
+
+        if overrides is not None:
+            self._request.update(overrides)
+
+    def add_veridian_tags(self, parameters: typing.Mapping, overrides: typing.Mapping = None):
+        manifest = parameters['Manifest']
+
+        # add tags indicating we want to get the Rap,Ticket
+        self._request['@BBTicket'] = True
+        self._request['@BMU,Ticket'] = True
+
+        self._request['BMU,ChipID'] = parameters['ChipID']
+        self._request['BMU,UniqueID'] = parameters['UniqueID']
+        self._request['BMU,ProductionMode'] = parameters['ProductionMode']
+
+        nonce = parameters.get('Nonce')
+        if nonce is not None:
+            self._request['BMU,Nonce'] = nonce
+
+        for comp_name, node in manifest.items():
+            if not comp_name.startswith('BMU,'):
+                continue
+
+            manifest_entry = dict(node)
+
+            # handle RestoreRequestRules
+            rules = manifest_entry['Info'].get('RestoreRequestRules')
+            if rules is not None:
+                self.apply_restore_request_rules(manifest_entry, parameters, rules)
+
+            # Make sure we have a Digest key for Trusted items even if empty
+            trusted = manifest_entry.get('Trusted', False)
+
+            if trusted:
+                digest = manifest_entry.get('Digest')
+                if digest is None:
+                    logging.debug(f'No Digest data, using empty value for entry {comp_name}')
+                    manifest_entry['Digest'] = b''
+
+            manifest_entry.pop('Info')
+
+            # finally add entry to request
+            self._request[comp_name] = manifest_entry
+
+        if overrides is not None:
+            self._request.update(overrides)
+
+    def img4_create_local_manifest(self, build_identity=None):
+        manifest = None
+        if build_identity is not None:
+            manifest = build_identity['Manifest']
+
         p = asn1.Encoder()
         p.start()
 
@@ -374,9 +535,17 @@ class TSSRequest:
         # now write the components
         for k, v in self._request.items():
             if isinstance(v, dict):
-                comp = img4_get_component_tag(k)
+                # check if component has Img4PayloadType
+                comp = None
+                if manifest is not None:
+                    comp = manifest[k]['Info'].get('Img4PayloadType')
+
+                if comp is None:
+                    comp = img4_get_component_tag(k)
+
                 if comp is None:
                     raise NotImplementedError(f'Unhandled component {k} - can\'t create manifest')
+
                 logging.debug(f'found component {comp} ({k})')
 
         # write manifest body header
@@ -407,8 +576,9 @@ class TSSRequest:
             request_script_path = TICKETS_SUBDIR / f'request_script.sh'
             response_path = TICKETS_SUBDIR / f'response_{unique_identifier}.txt'
 
-            if not request_plist_path.parent.exists():
-                request_plist_path.parent.mkdir()
+            TICKETS_SUBDIR.mkdir(parents=True, exist_ok=True)
+            for file in TICKETS_SUBDIR.iterdir():
+                file.unlink()
 
             with request_plist_path.open('wb') as f:
                 plistlib.dump(self._request, f)
