@@ -1,14 +1,13 @@
-from re import sub
-import plistlib
 import logging
-import struct
+import plistlib
 import ssl
+import struct
+from re import sub
 
 import IPython
 from pygments import highlight, lexers, formatters
-
 from pymobiledevice3 import usbmux
-from pymobiledevice3.exceptions import ConnectionFailedError, PyMobileDevice3Exception
+from pymobiledevice3.exceptions import ConnectionFailedError, PyMobileDevice3Exception, StartServiceError
 
 SHELL_USAGE = """
 # This shell allows you to communicate directly with every service layer behind the lockdownd daemon.
@@ -34,19 +33,15 @@ class ServiceConnection(object):
 
     @staticmethod
     def create(udid, port):
-        mux = usbmux.USBMux()
-        mux.process()
         target_device = None
-
         while target_device is None:
-            mux.process()
-            for connected_device in mux.devices:
-                if connected_device.serial == udid:
-                    target_device = connected_device
-                    break
+            matching_devices = [device for device in usbmux.list_devices() if device.serial == udid]
+            if len(matching_devices) == 1:
+                target_device = matching_devices[0]
+                break
 
         try:
-            socket = mux.connect(target_device, port)
+            socket = target_device.connect(port)
         except usbmux.MuxException:
             raise ConnectionFailedError(f'Connection to device port {port} failed')
 
@@ -115,9 +110,18 @@ class ServiceConnection(object):
         return self.sendall(message + payload)
 
     def ssl_start(self, keyfile, certfile):
-        context = ssl.SSLContext(ssl.PROTOCOL_TLSv1)
-        context.load_cert_chain(certfile, keyfile)
-        self.socket = context.wrap_socket(self.socket)
+        protocols = [ssl.PROTOCOL_TLSv1_2, ssl.PROTOCOL_TLSv1_1, ssl.PROTOCOL_TLSv1]
+        for protocol in protocols:
+            # try most recent protocol first, then fallback to each older for each one that fails
+            context = ssl.SSLContext(protocol)
+            context.load_cert_chain(certfile, keyfile)
+            try:
+                self.socket = context.wrap_socket(self.socket)
+                break
+            except ssl.SSLError:
+                pass
+        else:
+            raise StartServiceError('failed to find correct protocol version')
 
     def shell(self):
         IPython.embed(
