@@ -7,6 +7,15 @@ class ProcessControl:
     def __init__(self, dvt):
         self._channel = dvt.make_channel(self.IDENTIFIER)
 
+    def signal(self, pid: int, sig: int):
+        """
+        Send signal to process
+        :param pid: PID of process to send signal.
+        :param sig: SIGNAL to send
+        """
+        self._channel.sendSignal_toPid_(MessageAux().append_obj(sig).append_obj(pid), expects_reply=True)
+        return self._channel.receive_plist()
+
     def kill(self, pid: int):
         """
         Kill a process.
